@@ -12,6 +12,7 @@ from astersay.backends.base import BaseBackend, Listener
 
 
 class VoskRecognizer(BaseBackend):
+    pause = True
 
     HERTZ_LIST = (48000, 16000, 8000)
     # Счётчик текущих предварительных результатов. Сбрасывается финальным.
@@ -71,6 +72,8 @@ class VoskRecognizer(BaseBackend):
         recv = ws.recv
         try:
             for chunk in listener.listen():
+                if self.pause:
+                    chunk = b''
                 send(chunk)
                 yield loads(recv())
         except Exception as e:
@@ -111,6 +114,7 @@ class VoskRecognizer(BaseBackend):
             # используем запомненный.
             if text_buffer is None:
                 text_buffer = get_text_buffer()
+                logger_info(_('Работаем с буфером %s'), text_buffer.key)
             logger_debug(_('Ответ Vosk: %s'), answer)
             if 'result' in answer:
                 text = answer['text']
