@@ -5,6 +5,7 @@
 #
 import json
 import re
+import sys
 from datetime import datetime
 from gettext import gettext as _
 from logging import getLogger
@@ -417,6 +418,8 @@ class Settings:
     def configure(self):
         self.work_dir = work_dir = abspath(self.work_dir or DEFAULT_WORK_DIR)
         create_dirs(work_dir)
+        # Каталог, где не только диалоги, но и импортируемые классы диалогов.
+        sys.path.append(path_join(work_dir, 'dialogs'))
 
         self.main = get_main_config(work_dir)
         self.logging = get_logging_config(work_dir)
@@ -432,6 +435,15 @@ class Settings:
         map(root.removeFilter, root.filters[:])
         dictConfig(logging)
 
+    def get_dialog_class(self):
+        """Возвращает кастомный или базовый класс диалога."""
+        name = self.dialog_scheme.get('dialog_class')
+        if not name:
+            name = self.main.get('dialog_class', 'astersay.dialog.BaseDialog')
+        cls = import_string(name)
+        logger.info(_('Используется класс диалога %r.'), cls)
+        return cls
+
     @property
     def dialog_scheme(self):
         """Динамическая загрузка внешних изменений модели диалога."""
