@@ -12,6 +12,7 @@ import re
 import subprocess
 import wave
 from collections import OrderedDict
+from datetime import datetime
 from gettext import gettext as _
 from os.path import splitext, basename
 from tempfile import NamedTemporaryFile
@@ -61,7 +62,7 @@ class TextBuffer(list):
         return 0
 
 
-def _make_say_params(saydata, variables):
+def make_say_params(saydata, variables):
     """Возвращает готовые аргументы для метода say()."""
     params = {'text': '', 'escape_digits': '', 'voice': ''}
 
@@ -86,7 +87,7 @@ def _make_say_params(saydata, variables):
     return params
 
 
-def _make_pause(data, key):
+def make_pause(data, key):
     try:
         pause = float(data.get(key, 0))
     except ValueError:
@@ -98,7 +99,7 @@ def _make_pause(data, key):
     return pause
 
 
-def _prepare_answer(answer, export_name, script, variables, source=None):
+def prepare_answer(answer, export_name, script, variables, source=None):
     export_name_source = export_name + '_source'
     if source is True:
         variables[export_name_source] = answer
@@ -139,7 +140,7 @@ def _prepare_answer(answer, export_name, script, variables, source=None):
     variables[export_name] = answer
 
 
-class Dialog:
+class BaseDialog:
     """Диалог робота с позвонившим на номер абонентом."""
     is_closed = False
     # Значение меняется экземпляром распознавателя в процессе считывания стрима.
@@ -233,12 +234,33 @@ class Dialog:
         text = self.text_from_text_buffer(mode, text_buffer)
         return regexp.search(text), text
 
-    def capture_text(self, stopwords, mode=1, text_buffer=None):
-        """Метод ищет в тексте буфера по регулярному выражению."""
+    def capture_text(self, stopwords, mode=1, text_buffer=None,
+                     second=0, maxsilence=10, minlength=10):
+        """Собирает речь вплоть до выявления каких-либо ограничений."""
+        if not text_buffer:
+            text_buffer = self.get_last_text_buffer()
         text = self.text_from_text_buffer(mode, text_buffer)
         stop = False
         if stopwords and stopwords.intersection(text.split(' ')):
             stop = True
+            log_debug('Сработало ограничение по стоп-слову.')
+        else:
+            _expire = self.silence_seconds >= maxsilence
+            _size = len(text) >= minlength
+            _final = text_buffer.seconds_after_final > 1
+            stop = _expire and _size and _final
+            if _expire:
+                log_debug('Сработало ограничение по тишине.')
+                if _size:
+                    log_debug('Сработало ограничение по длине.')
+                    if _final:
+                        log_debug('Сработало ограничение по финалу '
+                                  'распознавания.')
+                    else:
+                        log_debug('Ожидаем финальное распознавание.')
+                else:
+                    log_debug('Длина текста недостаточна для завершения.')
+
         return text, stop
 
     def text_processing(self, name, mode=1, text_buffer=None):
@@ -277,7 +299,8 @@ class Dialog:
         self._text_processing_text = text
         return result, text
 
-    def say(self, text: str, voice: str, escape_digits: str, nonblocking=False):
+    def say(self, text: str, voice: str, escape_digits: str, nonblocking=False,
+            reset_silense=True):
         if not text and not voice:
             log_error(_('Текст и файл голоса отсутствуют.'))
             return
@@ -320,7 +343,8 @@ class Dialog:
                 log_debug(_('Голос %s завершён.'), voice)
                 # После завершения любой речи робота мы должны сбросить счётчик
                 # молчания.
-                self._reset_silence = True
+                if reset_silense:
+                    self._reset_silence = True
 
         def runner():
             rest = duration
@@ -357,7 +381,7 @@ class Dialog:
             s = conf['phrases'][0]
             index = 1
         saydata = {'text': s}
-        self.say(**_make_say_params(saydata, variables))
+        self.say(**make_say_params(saydata, variables))
         return index
 
     def calc_silence_seconds(self):
@@ -428,6 +452,8 @@ class Dialog:
                     self._reset_silence = False
                     log_debug(_('Сброс счётчика молчания.'))
                 sleep(1)
+                if self.recognizer.pause:
+                    self._silence_from = int(time())
                 self.calc_silence_seconds()
             thread = self._calc_silence_thread
             self._calc_silence_thread = None
@@ -482,6 +508,7 @@ class Dialog:
         self.start()
         hangup = False
         error = None
+        started = datetime.utcnow()
         try:
             self.main_script()
         except AgiHangupError:
@@ -494,6 +521,7 @@ class Dialog:
             error = _('Ошибка в сценарии диалога.')
             log_error(error, exc_info=e)
         self.stop()
+        finished = datetime.utcnow()
 
         agi = self.agi
         try:
@@ -517,6 +545,9 @@ class Dialog:
             hangup=hangup,
             error=error,
             chain=self.chain,
+            started='%sZ' % started.isoformat(),
+            finished='%sZ' % finished.isoformat(),
+            duration=(finished - started).total_seconds(),
         )
         if filename:
             log_info(_('Диалог экспортирован в файл %s'), filename)
@@ -597,7 +628,6 @@ class Dialog:
         return res, plugin_data
 
     def main_script(self):
-        started = time()
         scheme = self.settings.dialog_scheme
         constants = scheme.get('constants', {})
         self.export = variables = OrderedDict()
@@ -629,7 +659,7 @@ class Dialog:
             say_debug = process.get('say_debug')
             if say_debug:
                 saydata = {'text': _('Запускаю скрипт %(name)r') % process}
-                self.say(**_make_say_params(saydata, variables))
+                self.say(**make_say_params(saydata, variables))
 
             result, speech = method(script, variables)
             log_debug('  RESULT %r', result)
@@ -662,18 +692,18 @@ class Dialog:
                 log_debug('  FINISH')
                 if say_debug:
                     saydata = {'text': _('Завершаю диалог.')}
-                    self.say(**_make_say_params(saydata, variables))
+                    self.say(**make_say_params(saydata, variables))
                 process = None
             elif result and success in program:
                 if say_debug:
                     saydata = {'text': _('Исходная речь. %s.') % speech}
-                    self.say(**_make_say_params(saydata, variables))
+                    self.say(**make_say_params(saydata, variables))
                     if result is True:
                         saydata = {'text': _('В результате получена истина.')}
                     else:
                         saydata = {
                             'text': _('Разультат обработки. %s.') % result}
-                    self.say(**_make_say_params(saydata, variables))
+                    self.say(**make_say_params(saydata, variables))
                 process = program[success]
             elif not result and fail in program:
                 if say_debug:
@@ -681,9 +711,9 @@ class Dialog:
                         saydata = {'text': _('Исходная речь. %s.') % speech}
                     else:
                         saydata = {'text': _('Исходная речь пуста.')}
-                    self.say(**_make_say_params(saydata, variables))
+                    self.say(**make_say_params(saydata, variables))
                     saydata = {'text': _('Результат отсутствует.')}
-                    self.say(**_make_say_params(saydata, variables))
+                    self.say(**make_say_params(saydata, variables))
                 process = program[fail]
             else:
                 try:
@@ -693,18 +723,13 @@ class Dialog:
             log_debug('    NEXT %s', process)
             # if say_debug and process:
             #     saydata = {'text': _('Следующий скрипт %(name)r') % process}
-            #     self.say(**_make_say_params(saydata, variables))
+            #     self.say(**make_say_params(saydata, variables))
 
             # Восстанавливаем константы. Это проще, чем городить лес кода.
             variables.update(constants)
             # Сохраняем цепочку вызовов скриптов.
             chain.append([name, result, speech])
 
-        finished = time()
-        variables['_time_start_'] = started
-        variables['_time_finish_'] = finished
-        variables['_time_duration_'] = finished - started
-
         log_debug(_('Главный скрипт завершён.'))
 
     def _agi_execute(self, commands, key):
@@ -765,7 +790,7 @@ class Dialog:
         assert export_name and isinstance(export_name, str)
 
         pauses = script.get('pauses', {})
-        _make_pause(pauses, 'before')
+        make_pause(pauses, 'before')
 
         commands = script.get('commands', {})
         self._agi_execute(commands, 'before')
@@ -778,10 +803,10 @@ class Dialog:
 
         saydata = script.get('say_start')
         if saydata:
-            _make_pause(pauses, 'before_say_start')
+            make_pause(pauses, 'before_say_start')
             self._agi_execute(commands, 'before_say_start')
-            self.say(**_make_say_params(saydata, variables))
-            _make_pause(pauses, 'after_say_start')
+            self.say(**make_say_params(saydata, variables))
+            make_pause(pauses, 'after_say_start')
             self._agi_execute(commands, 'after_say_start')
         say_start_duration = self._voice_duration
 
@@ -789,7 +814,8 @@ class Dialog:
 
         maxtime = float(script.get('maxtime', 600))
         minlength = int(script.get('minlength', 10))
-        silence = int(script.get('silence', 15))
+        # TODO: убрать в версии 0.20 параметр `silence`.
+        maxsilence = int(script.get('maxsilence', script.get('silence', 10)))
         stopwords = script.get('stopwords', [])
         log_debug(_('Ограничение по времени: %s секунд.'), maxtime)
         log_debug(_('Ограничение по длине: от %s символов.'), minlength)
@@ -798,7 +824,14 @@ class Dialog:
         stopwords = set(stopwords)
 
         func = self.capture_text
-        speech, stop = func(stopwords, mode=mode)
+        speech, stop = func(
+            stopwords,
+            mode=mode,
+            text_buffer=text_buffer,
+            second=0,
+            maxsilence=maxsilence,
+            minlength=minlength,
+        )
         if stop:
             log_debug(_('Первая попытка успешна. Текст: %s'), speech)
         else:
@@ -812,26 +845,14 @@ class Dialog:
             log_debug(_('Стоп-слова пока нет, ждём %d-ю секунду.'), second)
             second += 0.5
             sleep(0.5)
-            speech, stop = func(stopwords, mode=mode)
-            if stop:
-                log_debug('Сработало ограничение по стоп-слову.')
-            else:
-                silence_seconds = self.silence_seconds
-                _expire = silence_seconds >= silence
-                _size = len(speech) >= minlength
-                _final = text_buffer.seconds_after_final > 1
-                stop = _expire and _size and _final
-                if _expire:
-                    log_debug('Сработало ограничение по тишине.')
-                    if _size:
-                        log_debug('Сработало ограничение по длине.')
-                        if _final:
-                            log_debug('Сработало ограничение по финалу '
-                                      'распознавания.')
-                        else:
-                            log_debug('Ожидаем финальное распознавание.')
-                    else:
-                        log_debug('Длина текста недостаточна для завершения.')
+            speech, stop = func(
+                stopwords,
+                mode=mode,
+                text_buffer=text_buffer,
+                second=second,
+                maxsilence=maxsilence,
+                minlength=minlength,
+            )
 
             if self.is_stopped:
                 log_debug(_('Диалог завершился до приёма данных.'))
@@ -842,7 +863,7 @@ class Dialog:
         if speech:
             answer = speech
             log_debug(_('Ответ: %r'), answer)
-            _prepare_answer(
+            prepare_answer(
                 answer=answer, source=answer, export_name=export_name,
                 script=script, variables=variables)
 
@@ -856,23 +877,23 @@ class Dialog:
 
             saydata = script.get('say_success')
             if saydata:
-                _make_pause(pauses, 'before_say_success')
+                make_pause(pauses, 'before_say_success')
                 self._agi_execute(commands, 'before_say_success')
-                self.say(**_make_say_params(saydata, variables))
-                _make_pause(pauses, 'after_say_success')
+                self.say(**make_say_params(saydata, variables))
+                make_pause(pauses, 'after_say_success')
                 self._agi_execute(commands, 'after_say_success')
         else:
             answer = None
             log_debug(_('Ответа нет.'))
             saydata = script.get('say_fail')
             if saydata:
-                _make_pause(pauses, 'before_say_fail')
+                make_pause(pauses, 'before_say_fail')
                 self._agi_execute(commands, 'before_say_fail')
-                self.say(**_make_say_params(saydata, variables))
-                _make_pause(pauses, 'after_say_fail')
+                self.say(**make_say_params(saydata, variables))
+                make_pause(pauses, 'after_say_fail')
                 self._agi_execute(commands, 'after_say_fail')
 
-        _make_pause(pauses, 'after')
+        make_pause(pauses, 'after')
         self._agi_execute(commands, 'after')
         return answer, speech
 
@@ -884,7 +905,7 @@ class Dialog:
         assert export_name and isinstance(export_name, str)
 
         pauses = script.get('pauses', {})
-        _make_pause(pauses, 'before')
+        make_pause(pauses, 'before')
 
         commands = script.get('commands', {})
         self._agi_execute(commands, 'before')
@@ -896,10 +917,10 @@ class Dialog:
 
         saydata = script.get('say_start')
         if saydata:
-            _make_pause(pauses, 'before_say_start')
+            make_pause(pauses, 'before_say_start')
             self._agi_execute(commands, 'before_say_start')
-            self.say(**_make_say_params(saydata, variables))
-            _make_pause(pauses, 'after_say_start')
+            self.say(**make_say_params(saydata, variables))
+            make_pause(pauses, 'after_say_start')
             self._agi_execute(commands, 'after_say_start')
         say_start_duration = self._voice_duration
 
@@ -948,7 +969,7 @@ class Dialog:
                 length_names = int(script.get('length_names', 2))
                 answer = ' '.join(names[:length_names])
                 log_debug(_('Ответ: %r'), answer)
-                _prepare_answer(
+                prepare_answer(
                     answer=answer, source=answer, export_name=export_name,
                     script=script, variables=variables)
 
@@ -956,10 +977,10 @@ class Dialog:
 
                 saydata = script.get('say_success')
                 if saydata:
-                    _make_pause(pauses, 'before_say_success')
+                    make_pause(pauses, 'before_say_success')
                     self._agi_execute(commands, 'before_say_success')
-                    self.say(**_make_say_params(saydata, variables))
-                    _make_pause(pauses, 'after_say_success')
+                    self.say(**make_say_params(saydata, variables))
+                    make_pause(pauses, 'after_say_success')
                     self._agi_execute(commands, 'after_say_success')
                 break
             elif attempt < attempts_count - 1:
@@ -967,10 +988,10 @@ class Dialog:
                     self.make_text_buffer()
                 saydata = attempts.get('say')
                 if saydata:
-                    _make_pause(pauses, 'before_say_attempt')
+                    make_pause(pauses, 'before_say_attempt')
                     self._agi_execute(commands, 'before_say_attempt')
-                    self.say(**_make_say_params(saydata, variables))
-                    _make_pause(pauses, 'after_say_attempt')
+                    self.say(**make_say_params(saydata, variables))
+                    make_pause(pauses, 'after_say_attempt')
                     self._agi_execute(commands, 'after_say_attempt')
 
         self.stop_recognizing()
@@ -979,13 +1000,13 @@ class Dialog:
             log_debug(_('Ответа нет.'))
             saydata = script.get('say_fail')
             if saydata:
-                _make_pause(pauses, 'before_say_fail')
+                make_pause(pauses, 'before_say_fail')
                 self._agi_execute(commands, 'before_say_fail')
-                self.say(**_make_say_params(saydata, variables))
-                _make_pause(pauses, 'after_say_fail')
+                self.say(**make_say_params(saydata, variables))
+                make_pause(pauses, 'after_say_fail')
                 self._agi_execute(commands, 'after_say_fail')
 
-        _make_pause(pauses, 'after')
+        make_pause(pauses, 'after')
         self._agi_execute(commands, 'after')
         return names, speech
 
@@ -1005,7 +1026,7 @@ class Dialog:
         assert pattern and isinstance(pattern, str)
 
         pauses = script.get('pauses', {})
-        _make_pause(pauses, 'before')
+        make_pause(pauses, 'before')
 
         commands = script.get('commands', {})
         self._agi_execute(commands, 'before')
@@ -1017,10 +1038,10 @@ class Dialog:
 
         saydata = script.get('say_start')
         if saydata:
-            _make_pause(pauses, 'before_say_start')
+            make_pause(pauses, 'before_say_start')
             self._agi_execute(commands, 'before_say_start')
-            self.say(**_make_say_params(saydata, variables))
-            _make_pause(pauses, 'after_say_start')
+            self.say(**make_say_params(saydata, variables))
+            make_pause(pauses, 'after_say_start')
             self._agi_execute(commands, 'after_say_start')
         say_start_duration = self._voice_duration
 
@@ -1069,7 +1090,7 @@ class Dialog:
             if match:
                 answer = match.group()
                 log_debug(_('Ответ: %r'), answer)
-                _prepare_answer(
+                prepare_answer(
                     answer=answer, source=answer, export_name=export_name,
                     script=script, variables=variables)
 
@@ -1077,10 +1098,10 @@ class Dialog:
 
                 saydata = script.get('say_success')
                 if saydata:
-                    _make_pause(pauses, 'before_say_success')
+                    make_pause(pauses, 'before_say_success')
                     self._agi_execute(commands, 'before_say_success')
-                    self.say(**_make_say_params(saydata, variables))
-                    _make_pause(pauses, 'after_say_success')
+                    self.say(**make_say_params(saydata, variables))
+                    make_pause(pauses, 'after_say_success')
                     self._agi_execute(commands, 'after_say_success')
                 break
             elif attempt < attempts_count - 1:
@@ -1088,10 +1109,10 @@ class Dialog:
                     self.make_text_buffer()
                 saydata = attempts.get('say')
                 if saydata:
-                    _make_pause(pauses, 'before_say_attempt')
+                    make_pause(pauses, 'before_say_attempt')
                     self._agi_execute(commands, 'before_say_attempt')
-                    self.say(**_make_say_params(saydata, variables))
-                    _make_pause(pauses, 'after_say_attempt')
+                    self.say(**make_say_params(saydata, variables))
+                    make_pause(pauses, 'after_say_attempt')
                     self._agi_execute(commands, 'after_say_attempt')
 
         self.stop_recognizing()
@@ -1100,13 +1121,13 @@ class Dialog:
             log_debug(_('Ответа нет.'))
             saydata = script.get('say_fail')
             if saydata:
-                _make_pause(pauses, 'before_say_fail')
+                make_pause(pauses, 'before_say_fail')
                 self._agi_execute(commands, 'before_say_fail')
-                self.say(**_make_say_params(saydata, variables))
-                _make_pause(pauses, 'after_say_fail')
+                self.say(**make_say_params(saydata, variables))
+                make_pause(pauses, 'after_say_fail')
                 self._agi_execute(commands, 'after_say_fail')
 
-        _make_pause(pauses, 'after')
+        make_pause(pauses, 'after')
         self._agi_execute(commands, 'after')
         if script.get('fail_on_false'):
             answer = variables.get(export_name) in (
@@ -1126,7 +1147,7 @@ class Dialog:
         assert filename and isinstance(filename, str)
 
         pauses = script.get('pauses', {})
-        _make_pause(pauses, 'before')
+        make_pause(pauses, 'before')
 
         commands = script.get('commands', {})
         self._agi_execute(commands, 'before')
@@ -1138,10 +1159,10 @@ class Dialog:
 
         saydata = script.get('say_start')
         if saydata:
-            _make_pause(pauses, 'before_say_start')
+            make_pause(pauses, 'before_say_start')
             self._agi_execute(commands, 'before_say_start')
-            self.say(**_make_say_params(saydata, variables))
-            _make_pause(pauses, 'after_say_start')
+            self.say(**make_say_params(saydata, variables))
+            make_pause(pauses, 'after_say_start')
             self._agi_execute(commands, 'after_say_start')
         say_start_duration = self._voice_duration
 
@@ -1189,7 +1210,7 @@ class Dialog:
             if response:
                 answer = response
                 log_debug(_('Ответ: %r'), answer)
-                _prepare_answer(
+                prepare_answer(
                     answer=answer, source=speech, export_name=export_name,
                     script=script, variables=variables)
 
@@ -1197,10 +1218,10 @@ class Dialog:
 
                 saydata = script.get('say_success')
                 if saydata:
-                    _make_pause(pauses, 'before_say_success')
+                    make_pause(pauses, 'before_say_success')
                     self._agi_execute(commands, 'before_say_success')
-                    self.say(**_make_say_params(saydata, variables))
-                    _make_pause(pauses, 'after_say_success')
+                    self.say(**make_say_params(saydata, variables))
+                    make_pause(pauses, 'after_say_success')
                     self._agi_execute(commands, 'after_say_success')
                 break
             elif attempt < attempts_count - 1:
@@ -1208,10 +1229,10 @@ class Dialog:
                     self.make_text_buffer()
                 saydata = attempts.get('say')
                 if saydata:
-                    _make_pause(pauses, 'before_say_attempt')
+                    make_pause(pauses, 'before_say_attempt')
                     self._agi_execute(commands, 'before_say_attempt')
-                    self.say(**_make_say_params(saydata, variables))
-                    _make_pause(pauses, 'after_say_attempt')
+                    self.say(**make_say_params(saydata, variables))
+                    make_pause(pauses, 'after_say_attempt')
                     self._agi_execute(commands, 'after_say_attempt')
 
         self.stop_recognizing()
@@ -1220,13 +1241,13 @@ class Dialog:
             log_debug(_('Ответа нет.'))
             saydata = script.get('say_fail')
             if saydata:
-                _make_pause(pauses, 'before_say_fail')
+                make_pause(pauses, 'before_say_fail')
                 self._agi_execute(commands, 'before_say_fail')
-                self.say(**_make_say_params(saydata, variables))
-                _make_pause(pauses, 'after_say_fail')
+                self.say(**make_say_params(saydata, variables))
+                make_pause(pauses, 'after_say_fail')
                 self._agi_execute(commands, 'after_say_fail')
 
-        _make_pause(pauses, 'after')
+        make_pause(pauses, 'after')
         self._agi_execute(commands, 'after')
         return response, speech
 
@@ -1234,10 +1255,10 @@ class Dialog:
         assert script['type'] == 'simple_say'
 
         if 'export_name' in script and 'export_value' in script:
-            variables[script['export_name']] = str(script['export_value'])
+            variables[script['export_name']] = script['export_value']
 
         pauses = script.get('pauses', {})
-        _make_pause(pauses, 'before')
+        make_pause(pauses, 'before')
 
         commands = script.get('commands', {})
         self._agi_execute(commands, 'before')
@@ -1256,10 +1277,10 @@ class Dialog:
 
         saydata = script.get('say_start')
         if saydata:
-            _make_pause(pauses, 'before_say_start')
+            make_pause(pauses, 'before_say_start')
             self._agi_execute(commands, 'before_say_start')
-            self.say(**_make_say_params(saydata, variables))
-            _make_pause(pauses, 'after_say_start')
+            self.say(**make_say_params(saydata, variables))
+            make_pause(pauses, 'after_say_start')
             self._agi_execute(commands, 'after_say_start')
 
         plugin_name = script.get('plugin_after')
@@ -1267,7 +1288,26 @@ class Dialog:
             res, plugin_data = self.execute_plugin(plugin_name, result, '', variables)
             result = plugin_data.get('result')
 
-        _make_pause(pauses, 'after')
+        if result:
+            log_debug(_('Результат есть.'))
+            saydata = script.get('say_success')
+            if saydata:
+                make_pause(pauses, 'before_say_success')
+                self._agi_execute(commands, 'before_say_success')
+                self.say(**make_say_params(saydata, variables))
+                make_pause(pauses, 'after_say_success')
+                self._agi_execute(commands, 'after_say_success')
+        else:
+            log_debug(_('Результата нет.'))
+            saydata = script.get('say_fail')
+            if saydata:
+                make_pause(pauses, 'before_say_fail')
+                self._agi_execute(commands, 'before_say_fail')
+                self.say(**make_say_params(saydata, variables))
+                make_pause(pauses, 'after_say_fail')
+                self._agi_execute(commands, 'after_say_fail')
+
+        make_pause(pauses, 'after')
         self._agi_execute(commands, 'after')
 
         speech = self.text_from_text_buffer()
