@@ -27,9 +27,9 @@ class Pymorphy2Processor:
         for word in words_iterator(text):
             for p in parse(word):
                 if 'Name' in p.tag and p.score >= prob_thresh:
-                    L.append(p.normal_form)
+                    L.append(p.normal_form.title())
                     break
                 elif 'Patr' in p.tag and p.score >= prob_thresh:
-                    L.append(word)
+                    L.append(word.title())
                     break
         return L
